@@ -7,21 +7,19 @@ Description of what this model does
 import os
 import hashlib
 import pathlib
+import importlib
 from typing import AsyncIterator, Tuple, Any, List, Type
 
-import numpy as np
-import pandas as pd
 
 # should be set before importing tensorflow
 os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"
-import tensorflow as tf
 
 from dffml.record import Record
 from dffml.model.accuracy import Accuracy
-from dffml.source.source import Sources
 from dffml.util.entrypoint import entrypoint
 from dffml.base import config, field
 from dffml.feature.feature import Feature, Features
+from dffml.source.source import Sources, SourcesContext
 from dffml.model.model import ModelContext, Model, ModelNotTrained
 from dffml_model_tensorflow.util.config.tensorflow import parse_layers
 
@@ -40,6 +38,7 @@ class TextClassifierConfig:
     predict: Feature = field("Feature name holding classification value")
     classifications: List[str] = field("Options for value of classification")
     features: Features = field("Features to train on")
+    directory: pathlib.Path = field("Directory where state should be saved")
     trainable: str = field(
         "Tweak pretrained model by training again", default=True
     )
@@ -68,10 +67,6 @@ class TextClassifierConfig:
     epochs: int = field(
         "Number of iterations to pass over all records in a source", default=10
     )
-    directory: pathlib.Path = field(
-        "Directory where state should be saved",
-        default=pathlib.Path("~", ".cache", "dffml", "tensorflow_hub"),
-    )
 
     def __post_init__(self):
         self.classifications = list(map(self.clstype, self.classifications))
@@ -87,6 +82,9 @@ class TextClassifierContext(ModelContext):
 
     def __init__(self, parent) -> None:
         super().__init__(parent)
+        self.tf = importlib.import_module("tensorflow")
+        self.np = importlib.import_module("numpy")
+        self.pd = importlib.import_module("pandas")
         self.cids = self._mkcids(self.parent.config.classifications)
         self.classifications = self._classifications(self.cids)
         self.features = self._applicable_features()
@@ -97,7 +95,7 @@ class TextClassifierContext(ModelContext):
         path = self._model_dir_path()
         if os.path.isfile(os.path.join(path, "saved_model.pb")):
             self.logger.info(f"Using saved model from {path}")
-            self._model = tf.keras.models.load_model(os.path.join(path))
+            self._model = self.tf.keras.models.load_model(os.path.join(path))
         else:
             self._model = self.createModel()
         return self
@@ -107,7 +105,7 @@ class TextClassifierContext(ModelContext):
 
     @property
     def classification(self):
-        return self.parent.config.predict.NAME
+        return self.parent.config.predict.name
 
     def _applicable_features(self):
         return [name for name in self.parent.config.features.names()]
@@ -199,15 +197,15 @@ class TextClassifierContext(ModelContext):
                 all_records.append(record)
         for record in all_records:
             for feature, results in record.features(self.features).items():
-                x_cols[feature].append(np.array(results))
+                x_cols[feature].append(self.np.array(results))
             y_cols.append(
                 self.classifications[record.feature(self.classification)]
             )
         if not y_cols:
             raise ValueError("No records to train on")
-        y_cols = np.array(y_cols)
+        y_cols = self.np.array(y_cols)
         for feature in x_cols:
-            x_cols[feature] = np.array(x_cols[feature])
+            x_cols[feature] = self.np.array(x_cols[feature])
         self.logger.info("------ Record Data ------")
         self.logger.info("x_cols:    %d", len(list(x_cols.values())[0]))
         self.logger.info("y_cols:    %d", len(y_cols))
@@ -269,7 +267,7 @@ class TextClassifierContext(ModelContext):
             batch_size=self.parent.config.batch_size,
             verbose=1,
         )
-        tf.keras.models.save_model(self._model, self._model_dir_path())
+        self.tf.keras.models.save_model(self._model, self._model_dir_path())
 
     async def accuracy(self, sources: Sources) -> Accuracy:
         """
@@ -285,7 +283,7 @@ class TextClassifierContext(ModelContext):
         return Accuracy(accuracy_score[1])
 
     async def predict(
-        self, records: AsyncIterator[Record]
+        self, sources: SourcesContext
     ) -> AsyncIterator[Tuple[Record, Any, float]]:
         """
         Uses trained data to make a prediction about the quality of a record.
@@ -295,17 +293,19 @@ class TextClassifierContext(ModelContext):
         ):
             raise ModelNotTrained("Train model before assessing for accuracy.")
 
-        async for record in records:
+        async for record in sources.with_features(self.features):
             feature_data = record.features(self.features)
-            df = pd.DataFrame(feature_data, index=[0])
-            predict = await self.prediction_data_generator(np.array(df)[0])
+            df = self.pd.DataFrame(feature_data, index=[0])
+            predict = await self.prediction_data_generator(
+                self.np.array(df)[0]
+            )
             all_prob = self._model.predict(predict)
             max_prob_idx = all_prob.argmax(axis=-1)
-            target = self.parent.config.predict.NAME
+            target = self.parent.config.predict.name
             self.logger.debug(
                 "Predicted probability of {} for {}: {}".format(
-                    self.parent.config.predict.NAME,
-                    np.array(df)[0],
+                    self.parent.config.predict.name,
+                    self.np.array(df)[0],
                     all_prob[0],
                 )
             )
@@ -323,57 +323,33 @@ class TextClassificationModel(Model):
     """
     Implemented using Tensorflow hub pretrained models.
 
-    .. code-block:: console
+    
+    .. literalinclude:: /../model/tensorflow_hub/examples/tfhub_text_classifier/train_data.sh
+
+    .. literalinclude:: /../model/tensorflow_hub/examples/tfhub_text_classifier/test_data.sh
+
+    Train the model
+
+    .. literalinclude:: /../model/tensorflow_hub/examples/tfhub_text_classifier/train.sh
+
+    Assess the accuracy
+
+    .. literalinclude:: /../model/tensorflow_hub/examples/tfhub_text_classifier/accuracy.sh
+
+    Output
+
+    .. code-block::
+
+        0.5
+
+    Make a prediction
+
+    .. literalinclude:: /../model/tensorflow_hub/examples/tfhub_text_classifier/predict.sh
+
+    Output
+
+    .. code-block:: json
 
-        $ cat > train.csv << EOF
-        sentence,sentiment
-        Life is good,1
-        This book is amazing,1
-        It's a terrible movie,0
-        Global warming is bad,0
-        EOF
-        $ cat > test.csv << EOF
-        sentence,sentiment
-        I am not feeling good,0
-        Our trip was full of adventures,1
-        EOF
-        $ dffml train \\
-            -model text_classifier \\
-            -model-epochs 30 \\
-            -model-predict sentiment:int:1 \\
-            -model-classifications 0 1  \\
-            -model-clstype int \\
-            -sources f=csv \\
-            -source-filename train.csv \\
-            -model-features \\
-              sentence:str:1 \\
-            -model-model_path "https://tfhub.dev/google/tf2-preview/gnews-swivel-20dim-with-oov/1" \\
-            -model-add_layers \\
-            -model-layers "Dense(units=512, activation='relu')" "Dense(units=2, activation='softmax')" \\
-            -log debug
-        $ dffml accuracy \\
-            -model text_classifier \\
-            -model-predict sentiment:int:1 \\
-            -model-classifications 0 1 \\
-            -model-model_path "https://tfhub.dev/google/tf2-preview/gnews-swivel-20dim-with-oov/1" \\
-            -model-clstype int \\
-            -sources f=csv \\
-            -source-filename test.csv \\
-            -model-features \\
-              sentence:str:1 \\
-            -log critical
-            1.0
-        $ dffml predict all \\
-            -model text_classifier \\
-            -model-predict sentiment:int:1 \\
-            -model-classifications 0 1 \\
-            -model-model_path "https://tfhub.dev/google/tf2-preview/gnews-swivel-20dim-with-oov/1" \\
-            -model-clstype int \\
-            -sources f=csv \\
-            -source-filename test.csv \\
-            -model-features \\
-              sentence:str:1 \\
-            -log debug
         [
             {
                 "extra": {},
@@ -382,11 +358,11 @@ class TextClassificationModel(Model):
                     "sentiment": 0
                 },
                 "key": "0",
-                "last_updated": "2020-02-15T02:54:02Z",
+                "last_updated": "2020-05-14T20:14:30Z",
                 "prediction": {
                     "sentiment": {
-                        "confidence": 0.7630850076675415,
-                        "value": 0
+                        "confidence": 0.9999992847442627,
+                        "value": 1
                     }
                 }
             },
@@ -397,16 +373,21 @@ class TextClassificationModel(Model):
                     "sentiment": 1
                 },
                 "key": "1",
-                "last_updated": "2020-02-15T02:54:02Z",
+                "last_updated": "2020-05-14T20:14:30Z",
                 "prediction": {
                     "sentiment": {
-                        "confidence": 0.6673157811164856,
+                        "confidence": 0.9999088048934937,
                         "value": 1
                     }
                 }
             }
         ]
 
+
+
+    Example usage of Tensorflow_hub Text Classifier model using python API
+
+    .. literalinclude:: /../model/tensorflow_hub/examples/tfhub_text_classifier/textclassifier.py
     """
 
     CONTEXT = TextClassifierContext
